@@ -74,7 +74,8 @@ variable: an unreadable object representing the error is printed instead.")
         (*print-level* nil)
         (*print-lines* nil)
         (*print-miser-width* nil)
-        (*print-pprint-dispatch* sb-pretty::*standard-pprint-dispatch-table*)
+        (*print-pprint-dispatch* (truly-the sb-pretty:pprint-dispatch-table
+                                            sb-pretty::*standard-pprint-dispatch-table*))
         (*print-pretty* nil)
         (*print-radix* nil)
         (*print-readably* t)
@@ -1909,7 +1910,7 @@ variable: an unreadable object representing the error is printed instead.")
              (write-string "broken weak pointer" stream))))))
 
 (defmethod print-object ((component code-component) stream)
-  (print-unreadable-object (component stream :identity t)
+  (print-unreadable-object (component stream)
     (let (dinfo)
       (cond ((eq (setq dinfo (%code-debug-info component)) :bpt-lra)
              (write-string "bpt-trap-return" stream))
@@ -1925,7 +1926,10 @@ variable: an unreadable object representing the error is printed instead.")
                (cond ((not (typep dinfo 'sb-c::debug-info)))
                      ((neq (sb-c::debug-info-name dinfo) fun-name)
                       (write-string ", " stream)
-                      (output-object (sb-c::debug-info-name dinfo) stream)))))))))
+                      (output-object (sb-c::debug-info-name dinfo) stream)))))))
+    (let ((a (get-lisp-obj-address component)))
+      (format stream " {~X..~X}"
+              a (+ (logandc2 a sb-vm:lowtag-mask) (code-object-size component))))))
 
 #-(or x86 x86-64 arm64 riscv)
 (defmethod print-object ((lra lra) stream)
